@@ -54,11 +54,22 @@ ZEND_BEGIN_ARG_INFO_EX(twig_template_get_attribute_args, ZEND_SEND_BY_VAL, ZEND_
 	ZEND_ARG_INFO(0, isDefinedTest)
 ZEND_END_ARG_INFO()
 
-zend_function_entry twig_functions[] = {
+#ifndef PHP_FE_END
+#define PHP_FE_END { NULL, NULL, NULL}
+#endif
+
+static const zend_function_entry twig_functions[] = {
 	PHP_FE(twig_template_get_attributes, twig_template_get_attribute_args)
-	{NULL, NULL, NULL}
+	PHP_FE_END
 };
 
+PHP_RSHUTDOWN_FUNCTION(twig)
+{
+#if ZEND_DEBUG
+	CG(unclean_shutdown) = 0; /* get rid of PHPUnit's exit() and report memleaks */
+#endif
+	return SUCCESS;
+}
 
 zend_module_entry twig_module_entry = {
 	STANDARD_MODULE_HEADER,
@@ -67,7 +78,7 @@ zend_module_entry twig_module_entry = {
 	NULL,
 	NULL,
 	NULL,
-	NULL,
+	PHP_RSHUTDOWN(twig),
 	NULL,
 	PHP_TWIG_VERSION,
 	STANDARD_MODULE_PROPERTIES
@@ -78,7 +89,7 @@ zend_module_entry twig_module_entry = {
 ZEND_GET_MODULE(twig)
 #endif
 
-int TWIG_ARRAY_KEY_EXISTS(zval *array, zval *key)
+static int TWIG_ARRAY_KEY_EXISTS(zval *array, zval *key)
 {
 	if (Z_TYPE_P(array) != IS_ARRAY) {
 		return 0;
@@ -100,7 +111,7 @@ int TWIG_ARRAY_KEY_EXISTS(zval *array, zval *key)
 	}
 }
 
-int TWIG_INSTANCE_OF(zval *object, zend_class_entry *interface TSRMLS_DC)
+static int TWIG_INSTANCE_OF(zval *object, zend_class_entry *interface TSRMLS_DC)
 {
 	if (Z_TYPE_P(object) != IS_OBJECT) {
 		return 0;
@@ -108,7 +119,7 @@ int TWIG_INSTANCE_OF(zval *object, zend_class_entry *interface TSRMLS_DC)
 	return instanceof_function(Z_OBJCE_P(object), interface TSRMLS_CC);
 }
 
-int TWIG_INSTANCE_OF_USERLAND(zval *object, char *interface TSRMLS_DC)
+static int TWIG_INSTANCE_OF_USERLAND(zval *object, char *interface TSRMLS_DC)
 {
 	zend_class_entry **pce;
 	if (Z_TYPE_P(object) != IS_OBJECT) {
@@ -120,7 +131,7 @@ int TWIG_INSTANCE_OF_USERLAND(zval *object, char *interface TSRMLS_DC)
 	return instanceof_function(Z_OBJCE_P(object), *pce TSRMLS_CC);
 }
 
-zval *TWIG_GET_ARRAYOBJECT_ELEMENT(zval *object, zval *offset TSRMLS_DC)
+static zval *TWIG_GET_ARRAYOBJECT_ELEMENT(zval *object, zval *offset TSRMLS_DC)
 {
 	zend_class_entry *ce = Z_OBJCE_P(object);
 	zval *retval;
@@ -143,7 +154,7 @@ zval *TWIG_GET_ARRAYOBJECT_ELEMENT(zval *object, zval *offset TSRMLS_DC)
 	return NULL;
 }
 
-int TWIG_ISSET_ARRAYOBJECT_ELEMENT(zval *object, zval *offset TSRMLS_DC)
+static int TWIG_ISSET_ARRAYOBJECT_ELEMENT(zval *object, zval *offset TSRMLS_DC)
 {
 	zend_class_entry *ce = Z_OBJCE_P(object);
 	zval *retval;
@@ -166,7 +177,7 @@ int TWIG_ISSET_ARRAYOBJECT_ELEMENT(zval *object, zval *offset TSRMLS_DC)
 	return 0;
 }
 
-char *TWIG_STRTOLOWER(const char *str, int str_len)
+static char *TWIG_STRTOLOWER(const char *str, int str_len)
 {
 	char *item_dup;
 
@@ -175,7 +186,7 @@ char *TWIG_STRTOLOWER(const char *str, int str_len)
 	return item_dup;
 }
 
-zval *TWIG_CALL_USER_FUNC_ARRAY(zval *object, char *function, zval *arguments TSRMLS_DC)
+static zval *TWIG_CALL_USER_FUNC_ARRAY(zval *object, char *function, zval *arguments TSRMLS_DC)
 {
 	zend_fcall_info fci;
 	zval ***args = NULL;
@@ -227,7 +238,7 @@ zval *TWIG_CALL_USER_FUNC_ARRAY(zval *object, char *function, zval *arguments TS
 	return retval_ptr;
 }
 
-int TWIG_CALL_BOOLEAN(zval *object, char *functionName TSRMLS_DC)
+static int TWIG_CALL_BOOLEAN(zval *object, char *functionName TSRMLS_DC)
 {
 	zval *ret;
 	int   res;
@@ -238,7 +249,7 @@ int TWIG_CALL_BOOLEAN(zval *object, char *functionName TSRMLS_DC)
 	return res;
 }
 
-zval *TWIG_GET_STATIC_PROPERTY(zval *class, char *prop_name TSRMLS_DC)
+static zval *TWIG_GET_STATIC_PROPERTY(zval *class, char *prop_name TSRMLS_DC)
 {
 	zval **tmp_zval;
 	zend_class_entry *ce;
@@ -256,7 +267,7 @@ zval *TWIG_GET_STATIC_PROPERTY(zval *class, char *prop_name TSRMLS_DC)
 	return *tmp_zval;
 }
 
-zval *TWIG_GET_ARRAY_ELEMENT_ZVAL(zval *class, zval *prop_name TSRMLS_DC)
+static zval *TWIG_GET_ARRAY_ELEMENT_ZVAL(zval *class, zval *prop_name TSRMLS_DC)
 {
 	zval **tmp_zval;
 
@@ -288,7 +299,7 @@ zval *TWIG_GET_ARRAY_ELEMENT_ZVAL(zval *class, zval *prop_name TSRMLS_DC)
 	return NULL;
 }
 
-zval *TWIG_GET_ARRAY_ELEMENT(zval *class, char *prop_name, int prop_name_length TSRMLS_DC)
+static zval *TWIG_GET_ARRAY_ELEMENT(zval *class, char *prop_name, int prop_name_length TSRMLS_DC)
 {
 	zval **tmp_zval;
 
@@ -314,7 +325,7 @@ zval *TWIG_GET_ARRAY_ELEMENT(zval *class, char *prop_name, int prop_name_length
 	return NULL;
 }
 
-zval *TWIG_PROPERTY(zval *object, zval *propname TSRMLS_DC)
+static zval *TWIG_PROPERTY(zval *object, zval *propname TSRMLS_DC)
 {
 	zval *tmp = NULL;
 
@@ -331,7 +342,7 @@ zval *TWIG_PROPERTY(zval *object, zval *propname TSRMLS_DC)
 	return tmp;
 }
 
-int TWIG_HAS_PROPERTY(zval *object, zval *propname TSRMLS_DC)
+static int TWIG_HAS_PROPERTY(zval *object, zval *propname TSRMLS_DC)
 {
 	if (Z_OBJ_HT_P(object)->has_property) {
 #if PHP_VERSION_ID >= 50400
@@ -343,7 +354,7 @@ int TWIG_HAS_PROPERTY(zval *object, zval *propname TSRMLS_DC)
 	return 0;
 }
 
-int TWIG_HAS_DYNAMIC_PROPERTY(zval *object, char *prop, int prop_len TSRMLS_DC)
+static int TWIG_HAS_DYNAMIC_PROPERTY(zval *object, char *prop, int prop_len TSRMLS_DC)
 {
 	if (Z_OBJ_HT_P(object)->get_properties) {
 		return zend_hash_quick_exists(
@@ -356,7 +367,7 @@ int TWIG_HAS_DYNAMIC_PROPERTY(zval *object, char *prop, int prop_len TSRMLS_DC)
 	return 0;
 }
 
-zval *TWIG_PROPERTY_CHAR(zval *object, char *propname TSRMLS_DC)
+static zval *TWIG_PROPERTY_CHAR(zval *object, char *propname TSRMLS_DC)
 {
 	zval *tmp_name_zval, *tmp;
 
@@ -367,12 +378,7 @@ zval *TWIG_PROPERTY_CHAR(zval *object, char *propname TSRMLS_DC)
 	return tmp;
 }
 
-int TWIG_CALL_B_0(zval *object, char *method)
-{
-	return 0;
-}
-
-zval *TWIG_CALL_S(zval *object, char *method, char *arg0 TSRMLS_DC)
+static zval *TWIG_CALL_S(zval *object, char *method, char *arg0 TSRMLS_DC)
 {
 	zend_fcall_info fci;
 	zval **args[1];
@@ -410,7 +416,7 @@ zval *TWIG_CALL_S(zval *object, char *method, char *arg0 TSRMLS_DC)
 	return retval_ptr;
 }
 
-int TWIG_CALL_SB(zval *object, char *method, char *arg0 TSRMLS_DC)
+static int TWIG_CALL_SB(zval *object, char *method, char *arg0 TSRMLS_DC)
 {
 	zval *retval_ptr;
 	int success;
@@ -425,51 +431,7 @@ int TWIG_CALL_SB(zval *object, char *method, char *arg0 TSRMLS_DC)
 	return success;
 }
 
-int TWIG_CALL_Z(zval *object, char *method, zval *arg1 TSRMLS_DC)
-{
-	zend_fcall_info fci;
-	zval **args[1];
-	zval *zfunction;
-	zval *retval_ptr;
-	int   success;
-
-	args[0] = &arg1;
-
-	MAKE_STD_ZVAL(zfunction);
-	ZVAL_STRING(zfunction, method, 1);
-	fci.size = sizeof(fci);
-	fci.function_table = EG(function_table);
-	fci.function_name = zfunction;
-	fci.symbol_table = NULL;
-#if PHP_VERSION_ID >= 50300
-	fci.object_ptr = object;
-#else
-	fci.object_pp = &object;
-#endif
-	fci.retval_ptr_ptr = &retval_ptr;
-	fci.param_count = 1;
-	fci.params = args;
-	fci.no_separation = 0;
-
-	if (zend_call_function(&fci, NULL TSRMLS_CC) == FAILURE) {
-		FREE_DTOR(zfunction);
-		if (retval_ptr) {
-			zval_ptr_dtor(&retval_ptr);
-		}
-		return 0;
-	}
-
-	FREE_DTOR(zfunction);
-
-	success = (retval_ptr && (Z_TYPE_P(retval_ptr) == IS_BOOL) && Z_LVAL_P(retval_ptr));
-	if (retval_ptr) {
-		zval_ptr_dtor(&retval_ptr);
-	}
-
-	return success;
-}
-
-int TWIG_CALL_ZZ(zval *object, char *method, zval *arg1, zval *arg2 TSRMLS_DC)
+static int TWIG_CALL_ZZ(zval *object, char *method, zval *arg1, zval *arg2 TSRMLS_DC)
 {
 	zend_fcall_info fci;
 	zval **args[2];
@@ -516,7 +478,7 @@ int TWIG_CALL_ZZ(zval *object, char *method, zval *arg1, zval *arg2 TSRMLS_DC)
 # define Z_UNSET_ISREF_P(pz) pz->is_ref = 0
 #endif
 
-void TWIG_NEW(zval *object, char *class, zval *arg0, zval *arg1 TSRMLS_DC)
+static void TWIG_NEW(zval *object, char *class, zval *arg0, zval *arg1 TSRMLS_DC)
 {
 	zend_class_entry **pce;
 
@@ -561,7 +523,7 @@ static int twig_add_array_key_to_string(void *pDest APPLY_TSRMLS_DC, int num_arg
 	return 0;
 }
 
-char *TWIG_IMPLODE_ARRAY_KEYS(char *joiner, zval *array TSRMLS_DC)
+static char *TWIG_IMPLODE_ARRAY_KEYS(char *joiner, zval *array TSRMLS_DC)
 {
 	smart_str collector = { 0, 0, 0 };
 
@@ -572,24 +534,6 @@ char *TWIG_IMPLODE_ARRAY_KEYS(char *joiner, zval *array TSRMLS_DC)
 	return collector.c;
 }
 
-static void TWIG_THROW_EXCEPTION(char *exception_name TSRMLS_DC, char *message, ...)
-{
-	char *buffer;
-	va_list args;
-	zend_class_entry **pce;
-
-	if (zend_lookup_class(exception_name, strlen(exception_name), &pce TSRMLS_CC) == FAILURE) {
-		return;
-	}
-
-	va_start(args, message);
-	vspprintf(&buffer, 0, message, args);
-	va_end(args);
-
-	zend_throw_exception_ex(*pce, 0 TSRMLS_CC, buffer);
-	efree(buffer);
-}
-
 static void TWIG_RUNTIME_ERROR(zval *template TSRMLS_DC, char *message, ...)
 {
 	char *buffer;
@@ -665,6 +609,7 @@ static char *TWIG_GET_CLASS_NAME(zval *object TSRMLS_DC)
 
 static int twig_add_method_to_class(void *pDest APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
+	zend_class_entry *ce;
 	zval *retval;
 	char *item;
 	size_t item_len;
@@ -675,12 +620,23 @@ static int twig_add_method_to_class(void *pDest APPLY_TSRMLS_DC, int num_args, v
 		return 0;
 	}
 
+	ce = *va_arg(args, zend_class_entry**);
 	retval = va_arg(args, zval*);
 
 	item_len = strlen(mptr->common.function_name);
 	item = estrndup(mptr->common.function_name, item_len);
 	php_strtolower(item, item_len);
 
+	if (strcmp("getenvironment", item) == 0) {
+		zend_class_entry **twig_template_ce;
+		if (zend_lookup_class("Twig_Template", strlen("Twig_Template"), &twig_template_ce TSRMLS_CC) == FAILURE) {
+			return 0;
+		}
+		if (instanceof_function(ce, *twig_template_ce TSRMLS_CC)) {
+			return 0;
+		}
+	}
+
 	add_assoc_stringl_ex(retval, item, item_len+1, item, item_len, 0);
 
 	return 0;
@@ -726,7 +682,7 @@ static void twig_add_class_to_cache(zval *cache, zval *object, char *class_name
 	array_init(class_methods);
 	array_init(class_properties);
 	// add all methods to self::cache[$class]['methods']
-	zend_hash_apply_with_arguments(&class_ce->function_table APPLY_TSRMLS_CC, twig_add_method_to_class, 1, class_methods);
+	zend_hash_apply_with_arguments(&class_ce->function_table APPLY_TSRMLS_CC, twig_add_method_to_class, 2, &class_ce, class_methods);
 	zend_hash_apply_with_arguments(&class_ce->properties_info APPLY_TSRMLS_CC, twig_add_property_to_class, 2, &class_ce, class_properties);
 
 	add_assoc_zval(class_info, "methods", class_methods);
@@ -794,6 +750,7 @@ PHP_FUNCTION(twig_template_get_attributes)
 		) {
 
 			if (isDefinedTest) {
+				efree(item);
 				RETURN_TRUE;
 			}
 
@@ -806,6 +763,7 @@ PHP_FUNCTION(twig_template_get_attributes)
 			if (free_ret) {
 				zval_ptr_dtor(&ret);
 			}
+			efree(item);
 			return;
 		}
 /*
@@ -819,9 +777,11 @@ PHP_FUNCTION(twig_template_get_attributes)
 */
 		if (strcmp("array", type) == 0 || Z_TYPE_P(object) != IS_OBJECT) {
 			if (isDefinedTest) {
+				efree(item);
 				RETURN_FALSE;
 			}
 			if (ignoreStrictCheck || !TWIG_CALL_BOOLEAN(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "isStrictVariables" TSRMLS_CC)) {
+				efree(item);
 				return;
 			}
 /*
@@ -831,12 +791,18 @@ PHP_FUNCTION(twig_template_get_attributes)
 				$message = sprintf('Impossible to access a key "%s" on an object of class "%s" that does not implement ArrayAccess interface', $item, get_class($object));
 			} elseif (is_array($object)) {
 				if (empty($object)) {
-				    $message = sprintf('Key "%s" does not exist as the array is empty', $arrayItem);
+					$message = sprintf('Key "%s" does not exist as the array is empty', $arrayItem);
 				} else {
-				    $message = sprintf('Key "%s" for array with keys "%s" does not exist', $arrayItem, implode(', ', array_keys($object)));
+					$message = sprintf('Key "%s" for array with keys "%s" does not exist', $arrayItem, implode(', ', array_keys($object)));
 				}
 			} elseif (Twig_Template::ARRAY_CALL === $type) {
-				$message = sprintf('Impossible to access a key ("%s") on a %s variable ("%s")', $item, gettype($object), $object);
+				if (null === $object) {
+					$message = sprintf('Impossible to access a key ("%s") on a null variable', $item);
+				} else {
+					$message = sprintf('Impossible to access a key ("%s") on a %s variable ("%s")', $item, gettype($object), $object);
+				}
+			} elseif (null === $object) {
+				$message = sprintf('Impossible to access an attribute ("%s") on a null variable', $item);
 			} else {
 				$message = sprintf('Impossible to access an attribute ("%s") on a %s variable ("%s")', $item, gettype($object), $object);
 			}
@@ -852,19 +818,31 @@ PHP_FUNCTION(twig_template_get_attributes)
 				if (0 == zend_hash_num_elements(Z_ARRVAL_P(object))) {
 					TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Key \"%s\" does not exist as the array is empty", item);
 				} else {
-					TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Key \"%s\" for array with keys \"%s\" does not exist", item, TWIG_IMPLODE_ARRAY_KEYS(", ", object TSRMLS_CC));
+					char *array_keys = TWIG_IMPLODE_ARRAY_KEYS(", ", object TSRMLS_CC);
+					TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Key \"%s\" for array with keys \"%s\" does not exist", item, array_keys);
+					efree(array_keys);
 				}
 			} else {
 				char *type_name = zend_zval_type_name(object);
 				Z_ADDREF_P(object);
-				convert_to_string(object);
-				TWIG_RUNTIME_ERROR(template TSRMLS_CC,
-					(strcmp("array", type) == 0)
-						? "Impossible to access a key (\"%s\") on a %s variable (\"%s\")"
-						: "Impossible to access an attribute (\"%s\") on a %s variable (\"%s\")",
-					item, type_name, Z_STRVAL_P(object));
+				if (Z_TYPE_P(object) == IS_NULL) {
+					convert_to_string(object);
+					TWIG_RUNTIME_ERROR(template TSRMLS_CC,
+						(strcmp("array", type) == 0)
+							? "Impossible to access a key (\"%s\") on a %s variable"
+							: "Impossible to access an attribute (\"%s\") on a %s variable",
+						item, type_name);
+				} else {
+					convert_to_string(object);
+					TWIG_RUNTIME_ERROR(template TSRMLS_CC,
+						(strcmp("array", type) == 0)
+							? "Impossible to access a key (\"%s\") on a %s variable (\"%s\")"
+							: "Impossible to access an attribute (\"%s\") on a %s variable (\"%s\")",
+						item, type_name, Z_STRVAL_P(object));
+				}
 				zval_ptr_dtor(&object);
 			}
+			efree(item);
 			return;
 		}
 	}
@@ -878,27 +856,42 @@ PHP_FUNCTION(twig_template_get_attributes)
 
 	if (Z_TYPE_P(object) != IS_OBJECT) {
 		if (isDefinedTest) {
+			efree(item);
 			RETURN_FALSE;
 		}
 /*
 		if ($ignoreStrictCheck || !$this->env->isStrictVariables()) {
 			return null;
 		}
-		throw new Twig_Error_Runtime(sprintf('Impossible to invoke a method ("%s") on a %s variable ("%s")', $item, gettype($object), $object), -1, $this->getTemplateName());
+
+		if (null === $object) {
+			$message = sprintf('Impossible to invoke a method ("%s") on a null variable', $item);
+		} else {
+			$message = sprintf('Impossible to invoke a method ("%s") on a %s variable ("%s")', $item, gettype($object), $object);
+		}
+
+		throw new Twig_Error_Runtime($message, -1, $this->getTemplateName());
 	}
 */
 		if (ignoreStrictCheck || !TWIG_CALL_BOOLEAN(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "isStrictVariables" TSRMLS_CC)) {
+			efree(item);
 			return;
 		}
 
 		type_name = zend_zval_type_name(object);
 		Z_ADDREF_P(object);
-		convert_to_string_ex(&object);
+		if (Z_TYPE_P(object) == IS_NULL) {
+			convert_to_string_ex(&object);
 
-		TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Impossible to invoke a method (\"%s\") on a %s variable (\"%s\")", item, type_name, Z_STRVAL_P(object));
+			TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Impossible to invoke a method (\"%s\") on a %s variable", item, type_name);
+		} else {
+			convert_to_string_ex(&object);
+
+			TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Impossible to invoke a method (\"%s\") on a %s variable (\"%s\")", item, type_name, Z_STRVAL_P(object));
+		}
 
 		zval_ptr_dtor(&object);
-
+		efree(item);
 		return;
 	}
 /*
@@ -917,7 +910,7 @@ PHP_FUNCTION(twig_template_get_attributes)
 
 /*
 	// object property
-	if (Twig_Template::METHOD_CALL !== $type) {
+	if (Twig_Template::METHOD_CALL !== $type && !$object instanceof Twig_Template) {
 		if (isset($object->$item) || array_key_exists((string) $item, $object)) {
 			if ($isDefinedTest) {
 				return true;
@@ -931,7 +924,7 @@ PHP_FUNCTION(twig_template_get_attributes)
 		}
 	}
 */
-	if (strcmp("method", type) != 0) {
+	if (strcmp("method", type) != 0 && !TWIG_INSTANCE_OF_USERLAND(object, "Twig_Template" TSRMLS_CC)) {
 		zval *tmp_properties, *tmp_item;
 
 		tmp_properties = TWIG_GET_ARRAY_ELEMENT(tmp_class, "properties", strlen("properties") TSRMLS_CC);
@@ -939,23 +932,42 @@ PHP_FUNCTION(twig_template_get_attributes)
 
 		if (tmp_item || TWIG_HAS_PROPERTY(object, zitem TSRMLS_CC) || TWIG_HAS_DYNAMIC_PROPERTY(object, item, item_len TSRMLS_CC)) {
 			if (isDefinedTest) {
+				efree(item);
 				RETURN_TRUE;
 			}
 			if (TWIG_CALL_SB(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "hasExtension", "sandbox" TSRMLS_CC)) {
 				TWIG_CALL_ZZ(TWIG_CALL_S(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "getExtension", "sandbox" TSRMLS_CC), "checkPropertyAllowed", object, zitem TSRMLS_CC);
 			}
 			if (EG(exception)) {
+				efree(item);
 				return;
 			}
 
 			ret = TWIG_PROPERTY(object, zitem TSRMLS_CC);
+			efree(item);
 			RETURN_ZVAL(ret, 1, 0);
 		}
 	}
 /*
 	// object method
 	if (!isset(self::$cache[$class]['methods'])) {
-		self::$cache[$class]['methods'] = array_change_key_case(array_flip(get_class_methods($object)));
+		if ($object instanceof self) {
+			$ref = new ReflectionClass($class);
+			$methods = array();
+
+			foreach ($ref->getMethods(ReflectionMethod::IS_PUBLIC) as $refMethod) {
+				$methodName = strtolower($refMethod->name);
+
+				// Accessing the environment from templates is forbidden to prevent untrusted changes to the environment
+				if ('getenvironment' !== $methodName) {
+					$methods[$methodName] = true;
+				}
+			}
+
+			self::$cache[$class]['methods'] = $methods;
+        } else {
+			self::$cache[$class]['methods'] = array_change_key_case(array_flip(get_class_methods($object)));
+        }
 	}
 
 	$call = false;
@@ -1021,19 +1033,22 @@ PHP_FUNCTION(twig_template_get_attributes)
 			efree(lcItem);
 
 			if (isDefinedTest) {
+				efree(item);
 				RETURN_FALSE;
 			}
 			if (ignoreStrictCheck || !TWIG_CALL_BOOLEAN(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "isStrictVariables" TSRMLS_CC)) {
+				efree(item);
 				return;
 			}
 			TWIG_RUNTIME_ERROR(template TSRMLS_CC, "Method \"%s\" for object \"%s\" does not exist", item, TWIG_GET_CLASS_NAME(object TSRMLS_CC));
+			efree(item);
 			return;
 		}
 
 		if (isDefinedTest) {
 			efree(tmp_method_name_get);
 			efree(tmp_method_name_is);
-			efree(lcItem);
+			efree(lcItem);efree(item);
 			RETURN_TRUE;
 		}
 /*
@@ -1046,11 +1061,11 @@ PHP_FUNCTION(twig_template_get_attributes)
 		if (TWIG_CALL_SB(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "hasExtension", "sandbox" TSRMLS_CC)) {
 			TWIG_CALL_ZZ(TWIG_CALL_S(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "getExtension", "sandbox" TSRMLS_CC), "checkMethodAllowed", object, zmethod TSRMLS_CC);
 		}
+		zval_ptr_dtor(&zmethod);
 		if (EG(exception)) {
 			efree(tmp_method_name_get);
 			efree(tmp_method_name_is);
-			efree(lcItem);
-			zval_ptr_dtor(&zmethod);
+			efree(lcItem);efree(item);
 			return;
 		}
 /*
@@ -1068,6 +1083,9 @@ PHP_FUNCTION(twig_template_get_attributes)
 		ret = TWIG_CALL_USER_FUNC_ARRAY(object, method, arguments TSRMLS_CC);
 		if (EG(exception) && TWIG_INSTANCE_OF(EG(exception), spl_ce_BadMethodCallException TSRMLS_CC)) {
 			if (ignoreStrictCheck || !TWIG_CALL_BOOLEAN(TWIG_PROPERTY_CHAR(template, "env" TSRMLS_CC), "isStrictVariables" TSRMLS_CC)) {
+				efree(tmp_method_name_get);
+				efree(tmp_method_name_is);
+				efree(lcItem);efree(item);
 				zend_clear_exception(TSRMLS_C);
 				return;
 			}
@@ -1076,7 +1094,6 @@ PHP_FUNCTION(twig_template_get_attributes)
 		efree(tmp_method_name_get);
 		efree(tmp_method_name_is);
 		efree(lcItem);
-		zval_ptr_dtor(&zmethod);
 	}
 /*
 	// useful when calling a template method from a template
@@ -1087,6 +1104,7 @@ PHP_FUNCTION(twig_template_get_attributes)
 
 	return $ret;
 */
+	efree(item);
 	// ret can be null, if e.g. the called method throws an exception
 	if (ret) {
 		if (TWIG_INSTANCE_OF_USERLAND(object, "Twig_TemplateInterface" TSRMLS_CC)) {
